@@ -26,8 +26,10 @@ use std::cmp::{Eq, Ord};
 use std::collections::hash_map::RandomState;
 use std::hash::{BuildHasher, Hash};
 use std::iter::Iterator;
+use std::mem::{replace, swap};
 
 use indexmap::map::{IndexMap, MutableKeys};
+use take_mut::take;
 
 /// A priority queue with efficient change function to change the priority of an
 /// element.
@@ -44,9 +46,9 @@ where
     I: Hash + Eq,
     P: Ord,
 {
-    pub(crate) map: IndexMap<I, Option<P>, H>, // Stores the items and assign them an index
-    heap: Vec<usize>,                          // Implements the heap of indexes
-    qp: Vec<usize>,                            // Performs the translation from the index
+    pub(crate) map: IndexMap<I, P, H>, // Stores the items and assign them an index
+    heap: Vec<usize>,                  // Implements the heap of indexes
+    qp: Vec<usize>,                    // Performs the translation from the index
     // of the map to the index of the heap
     size: usize, // The size of the heap
 }
@@ -143,7 +145,7 @@ where
     P: Ord,
     I: Hash + Eq,
 {
-    /// Return n iterator in arbitrary order over the
+    /// Return an iterator in arbitrary order over the
     /// (item, priority) elements in the queue.
     ///
     /// The item and the priority are mutable references, but it's a logic error
@@ -165,9 +167,7 @@ where
         if self.size == 0 {
             return None;
         }
-        self.map
-            .get_index(unsafe { *self.heap.get_unchecked(0) })
-            .map(|(k, v)| (k, v.as_ref().unwrap()))
+        self.map.get_index(unsafe { *self.heap.get_unchecked(0) })
     }
 
     /// Returns the couple (item, priority) with the greatest
@@ -187,7 +187,7 @@ where
         }
         self.map
             .get_index_mut(unsafe { *self.heap.get_unchecked(0) })
-            .map(|(k, v)| (k, v.as_ref().unwrap()))
+            .map(|(k, v)| (k, &*v))
     }
 
     /// Returns the number of elements the internal map can hold without
@@ -259,7 +259,7 @@ where
 {
     /// Insert the item-priority pair into the queue.
     ///
-    /// If an element equals to `item` was already into the queue,
+    /// If an element equal to `item` was already into the queue,
     /// it is updated and the old value of its priority returned in `Some`;
     /// otherwise, return `None`.
     ///
@@ -267,17 +267,15 @@ where
     pub fn push(&mut self, item: I, priority: P) -> Option<P> {
         use indexmap::map::Entry::*;
         let mut pos = 0;
-        let oldp;
+        let mut oldp = None;
 
         match self.map.entry(item) {
             Occupied(mut e) => {
-                oldp = e.get_mut().take();
-                *e.get_mut() = Some(priority);
+                oldp = Some(replace(e.get_mut(), priority));
                 pos = unsafe { *self.qp.get_unchecked(e.index()) };
             }
             Vacant(e) => {
-                e.insert(Some(priority));
-                oldp = None;
+                e.insert(priority);
             }
         }
 
@@ -334,6 +332,50 @@ where
         None
     }
 
+    /// Increase the priority of an existing item in the queue, or
+    /// insert it if not present.
+    ///
+    /// If an element equal to `item` is already in the queue with a
+    /// lower priority, its priority is increased to the new one
+    /// without replacing the element and the old priority is returned.
+    /// Otherwise, the new element is inserted into the queue.
+    ///
+    /// Returns `Some` if an element equal to `item` is already in the
+    /// queue. If its priority is higher then `priority`, the latter is returned back,
+    /// otherwise, the old priority is contained in the Option.
+    /// If the item is not in the queue, `None` is returned.
+    ///
+    /// Computes in **O(log(N))** time.
+    pub fn push_increase(&mut self, item: I, priority: P) -> Option<P> {
+        if self.get(&item).map_or(true, |(_, p)| priority > *p) {
+            self.push(item, priority)
+        } else {
+            Some(priority)
+        }
+    }
+
+    /// Decrease the priority of an existing item in the queue, or
+    /// insert it if not present.
+    ///
+    /// If an element equal to `item` is already in the queue with a
+    /// higher priority, its priority is decreased to the new one
+    /// without replacing the element and the old priority is returned.
+    /// Otherwise, the new element is inserted into the queue.
+    ///
+    /// Returns `Some` if an element equal to `item` is already in the
+    /// queue. If its priority is lower then `priority`, the latter is returned back,
+    /// otherwise, the old priority is contained in the Option.
+    /// If the item is not in the queue, `None` is returned.
+    ///
+    /// Computes in **O(log(N))** time.
+    pub fn push_decrease(&mut self, item: I, priority: P) -> Option<P> {
+        if self.get(&item).map_or(true, |(_, p)| priority < *p) {
+            self.push(item, priority)
+        } else {
+            Some(priority)
+        }
+    }
+
     /// Change the priority of an Item returning the old value of priority,
     /// or `None` if the item wasn't in the queue.
     ///
@@ -342,17 +384,16 @@ where
     ///
     /// The item is found in **O(1)** thanks to the hash table.
     /// The operation is performed in **O(log(N))** time.
-    pub fn change_priority<Q: ?Sized>(&mut self, item: &Q, new_priority: P) -> Option<P>
+    pub fn change_priority<Q: ?Sized>(&mut self, item: &Q, mut new_priority: P) -> Option<P>
     where
         I: Borrow<Q>,
         Q: Eq + Hash,
     {
         let mut pos;
         let r = if let Some((index, _, p)) = self.map.get_full_mut(item) {
-            let oldp = p.take();
-            *p = Some(new_priority);
+            swap(p, &mut new_priority);
             pos = unsafe { *self.qp.get_unchecked(index) };
-            oldp
+            Some(new_priority)
         } else {
             return None;
         };
@@ -391,8 +432,7 @@ where
         let mut pos = 0;
         let mut found = false;
         if let Some((index, _, p)) = self.map.get_full_mut(item) {
-            let oldp = p.take().unwrap();
-            *p = Some(priority_setter(oldp));
+            take(p, priority_setter);
             pos = unsafe { *self.qp.get_unchecked(index) };
             found = true;
         }
@@ -424,7 +464,7 @@ where
         I: Borrow<Q>,
         Q: Eq + Hash,
     {
-        self.map.get(item).map(|o| o.as_ref().unwrap())
+        self.map.get(item)
     }
 
     /// Get the couple (item, priority) of an arbitrary element, as reference
@@ -434,9 +474,7 @@ where
         I: Borrow<Q>,
         Q: Eq + Hash,
     {
-        self.map
-            .get_full(item)
-            .map(|(_, k, v)| (k, v.as_ref().unwrap()))
+        self.map.get_full(item).map(|(_, k, v)| (k, v))
     }
 
     /// Get the couple (item, priority) of an arbitrary element, or `None`
@@ -453,9 +491,7 @@ where
         I: Borrow<Q>,
         Q: Eq + Hash,
     {
-        self.map
-            .get_full_mut2(item)
-            .map(|(_, k, v)| (k, v.as_ref().unwrap()))
+        self.map.get_full_mut2(item).map(|(_, k, v)| (k, &*v))
     }
 
     /// Returns the items not ordered
@@ -560,10 +596,7 @@ where
         // swap remove the old heap from the qp
         if self.size == 0 {
             self.qp.pop();
-            return self
-                .map
-                .swap_remove_index(head)
-                .map(|(i, o)| (i, o.unwrap()));
+            return self.map.swap_remove_index(head);
         }
         unsafe {
             *self.qp.get_unchecked_mut(*self.heap.get_unchecked(0)) = 0;
@@ -575,9 +608,7 @@ where
             }
         }
         // swap remove from the map and return to the client
-        self.map
-            .swap_remove_index(head)
-            .map(|(i, o)| (i, o.unwrap()))
+        self.map.swap_remove_index(head)
     }
 
     /// Swap two elements keeping a consistent state.
@@ -669,7 +700,7 @@ where
         let mut i = 0;
         for (item, priority) in vec {
             if !pq.map.contains_key(&item) {
-                pq.map.insert(item, Some(priority));
+                pq.map.insert(item, priority);
                 pq.qp.push(i);
                 pq.heap.push(i);
                 i += 1;
@@ -706,9 +737,9 @@ where
             if pq.map.contains_key(&item) {
                 let (_, old_item, old_priority) = pq.map.get_full_mut2(&item).unwrap();
                 *old_item = item;
-                *old_priority = Some(priority);
+                *old_priority = priority;
             } else {
-                pq.map.insert(item, Some(priority));
+                pq.map.insert(item, priority);
                 pq.qp.push(pq.size);
                 pq.heap.push(pq.size);
                 pq.size += 1;
@@ -784,9 +815,9 @@ where
                 if self.map.contains_key(&item) {
                     let (_, old_item, old_priority) = self.map.get_full_mut2(&item).unwrap();
                     *old_item = item;
-                    *old_priority = Some(priority);
+                    *old_priority = priority;
                 } else {
-                    self.map.insert(item, Some(priority));
+                    self.map.insert(item, priority);
                     self.qp.push(self.size);
                     self.heap.push(self.size);
                     self.size += 1;
@@ -809,12 +840,7 @@ where
 {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_map()
-            .entries(
-                self.heap
-                    .iter()
-                    .map(|&i| self.map.get_index(i).unwrap())
-                    .map(|(i, op)| (i, op.as_ref().unwrap())),
-            )
+            .entries(self.heap.iter().map(|&i| self.map.get_index(i).unwrap()))
             .finish()
     }
 }
@@ -891,7 +917,7 @@ mod serde {
         {
             let mut map_serializer = serializer.serialize_seq(Some(self.size))?;
             for (k, v) in &self.map {
-                map_serializer.serialize_element(&(k, v.as_ref().unwrap()))?;
+                map_serializer.serialize_element(&(k, v))?;
             }
             map_serializer.end()
         }
@@ -948,7 +974,7 @@ mod serde {
             };
 
             while let Some((item, priority)) = seq.next_element()? {
-                pq.map.insert(item, Some(priority));
+                pq.map.insert(item, priority);
                 pq.qp.push(pq.size);
                 pq.heap.push(pq.size);
                 pq.size += 1;
